use async_graphql::dataloader::*;
use async_graphql::*;
use itertools::Itertools;
use std::collections::HashMap;
use std::sync::Arc;

use crate::domain::gruppe::model::Gruppe;
use crate::scalar::Id;

pub struct RollenGruppenLoader {
    pub pool: sqlx::PgPool,
}

impl Loader<Id> for RollenGruppenLoader {
    type Value = Vec<Gruppe>;
    type Error = Arc<sqlx::Error>;

    async fn load(&self, keys: &[Id]) -> Result<HashMap<Id, Self::Value>, Self::Error> {
        let rows = sqlx::query!(
            r#"
                SELECT
                    rg.rolle_id,
                    g.id,
                    g.gruppenname,
                    g.erstellt_am,
                    g.geaendert_am
                FROM gruppen AS g
                LEFT JOIN rollen_gruppen AS rg ON g.id = rg.gruppe_id
                WHERE rg.rolle_id = ANY($1);
            "#,
            keys
        )
        .fetch_all(&self.pool)
        .await?
        .into_iter()
        .map(|row| {
            (
                row.rolle_id,
                Gruppe {
                    id: row.id,
                    gruppenname: row.gruppenname,
                    erstellt_am: row.erstellt_am,
                    geaendert_am: row.geaendert_am,
                },
            )
        })
        .into_group_map();

        Ok(rows)
    }
}
