use anyhow::Error;

use super::Repository;
use crate::database::Queryer;
use crate::domain::benutzer::entity;
use crate::domain::benutzer::model;
use crate::scalar::Id;

impl Repository {
    pub async fn create_benutzer<'c, C: Queryer<'c>>(
        &self,
        db: C,
        benutzer: &entity::Benutzer,
    ) -> Result<model::Benutzer, Error> {
        const QUERY: &str = r#"
            INSERT INTO benutzer (id, kennung, nachname, vorname)
            VALUES ($1, $2, $3, $4)
            RETURNING id, kennung, nachname, vorname;
        "#;

        let benutzer = sqlx::query_as::<_, model::Benutzer>(QUERY)
            .bind::<Id>(benutzer.id)
            .bind(&benutzer.kennung)
            .bind(&benutzer.nachname)
            .bind(&benutzer.vorname)
            .fetch_one(db)
            .await?;

        Ok(benutzer)
    }
}
