use std::collections::HashMap;
use std::sync::Arc;

use itertools::Itertools;

use super::Repository;
use crate::database::Queryer;
use crate::domain::gruppe::model::Gruppe;
use crate::scalar::Id;

impl Repository {
    pub async fn gruppe_dataloader<'c, C: Queryer<'c>>(
        &self,
        db: C,
        keys: &[Id],
    ) -> Result<HashMap<Id, Vec<Gruppe>>, Arc<sqlx::Error>> {
        let rows = sqlx::query!(
            r#"
                SELECT
                    bg.benutzer_id,
                    g.id,
                    g.gruppenname,
                    g.erstellt_am,
                    g.geaendert_am
                FROM gruppen AS g
                LEFT JOIN benutzer_gruppen AS bg ON g.id = bg.gruppe_id
                WHERE bg.benutzer_id = ANY($1);
            "#,
            keys
        )
        .fetch_all(db)
        .await?
        .into_iter()
        .map(|row| {
            (
                row.benutzer_id,
                Gruppe {
                    id: row.id,
                    gruppenname: row.gruppenname,
                    erstellt_am: row.erstellt_am,
                    geaendert_am: row.geaendert_am,
                },
            )
        })
        .into_group_map();

        Ok(rows)
    }
}
