use anyhow::Error;

use super::Repository;
use crate::database::Queryer;
use crate::domain::gruppe::{entity, model};

impl Repository {
    pub async fn gruppe_erstellen<'c, C: Queryer<'c>>(
        &self,
        db: C,
        gruppe: &entity::Gruppe,
    ) -> Result<model::Gruppe, Error> {
        const QUERY: &str = r#"
            INSERT INTO gruppen (id, erstellt_am, geaendert_am, gruppenname) VALUES (
                $1, $2, $3, $4
            ) RETURNING id, erstellt_am, geaendert_am, gruppenname;
        "#;

        let gruppe = sqlx::query_as::<_, model::Gruppe>(QUERY)
            .bind(gruppe.id)
            .bind(gruppe.erstellt_am)
            .bind(gruppe.geaendert_am)
            .bind(&gruppe.gruppenname)
            .fetch_one(db)
            .await?;

        Ok(gruppe)
    }
}
