use anyhow::Error;

use super::Repository;
use crate::database::Queryer;
use crate::domain::gruppe::{entity, model};

impl Repository {
    pub async fn gruppe_update<'c, C: Queryer<'c>>(
        &self,
        db: C,
        gruppe: &entity::Gruppe,
    ) -> Result<model::Gruppe, Error> {
        const QUERY: &str = r#"
            UPDATE gruppen
            SET geaendert_am = $2, ruppenname = $3 WHERE gruppeid = $1
            RETURNING id, geaendert_am, gruppenname, gruppenname;
        "#;

        let gruppe = sqlx::query_as::<_, model::Gruppe>(QUERY)
            .bind(gruppe.id)
            .bind(gruppe.geaendert_am)
            .bind(&gruppe.gruppenname)
            .fetch_one(db)
            .await?;

        Ok(gruppe)
    }
}
