use async_graphql::{ComplexObject, Context, FieldResult, SimpleObject};

use crate::{
    dataloader::LoaderContext,
    domain::gruppe::model::Gruppe,
    scalar::{Id, Time},
};

/// Um die Administration zu erleichtern werden Gruppen in die Rollen hinzugefuegt
#[derive(sqlx::FromRow, SimpleObject, Debug, Clone)]
#[graphql(complex)]
pub struct Rolle {
    /// Die  uuid einer Rolle
    pub id: Id,

    /// Der Rollenname
    pub rollenname: String,

    /// Wann die Rolle erstellt wurde
    pub erstellt_am: Time,

    /// Wann die Rolle geaendert wurde
    pub geaendert_am: Time,
}

#[ComplexObject]
impl Rolle {
    /// Die Gruppen in einer Rolle
    pub async fn gruppen<'ctx>(&self, ctx: &Context<'ctx>) -> FieldResult<Option<Vec<Gruppe>>> {
        let loader = ctx.data::<LoaderContext>()?;
        Ok(loader.gruppen.load_one(self.id).await?)
    }
}
