use std::collections::HashMap;
use std::sync::Arc;

use itertools::Itertools;

use super::Repository;
use crate::database::Queryer;
use crate::domain::rolle::model::Rolle;
use crate::scalar::Id;

impl Repository {
    pub async fn rollen_dataloader<'c, C: Queryer<'c>>(
        &self,
        db: C,
        keys: &[Id],
    ) -> Result<HashMap<Id, Vec<Rolle>>, Arc<sqlx::Error>> {
        let rows = sqlx::query!(
            r#"
                SELECT
                    br.benutzer_id,
                    r.id,
                    r.rollenname,
                    r.erstellt_am,
                    r.geaendert_am
                FROM rollen AS r
                LEFT JOIN benutzer_rollen AS br ON r.id = br.rolle_id
                WHERE br.benutzer_id = ANY($1);
            "#,
            keys
        )
        .fetch_all(db)
        .await?
        .into_iter()
        .map(|row| {
            (
                row.benutzer_id,
                Rolle {
                    id: row.id,
                    rollenname: row.rollenname,
                    erstellt_am: row.erstellt_am,
                    geaendert_am: row.geaendert_am,
                },
            )
        })
        .into_group_map();

        Ok(rows)
    }
}
