use async_graphql::{ComplexObject, Enum, SimpleObject};
use sqlx::Type;

use crate::scalar::{Id, Time};

/// Aus welcher Quille die Gruppe stammt.
#[derive(Enum, Clone, Debug, Eq, Copy, PartialEq, Type)]
#[sqlx(type_name = "gruppen_herkunft", rename_all = "lowercase")]
pub enum Herkunft {
    /// Die Gruppe wurde direkt dem Benutzer zugefügt
    Direkt,

    /// Die Gruppe wurde durch eine Rolle vergeben
    Indirekt,

    /// Die Gruppe ist sowohl direkt als auch indirekt zugefügt
    Beides,
}

#[derive(sqlx::FromRow, SimpleObject, Clone, Debug)]
#[graphql(complex)]
pub struct GruppeAnsicht {
    /// Die UUID einer Gruppe
    pub id: Id,

    /// Der Gruppenname
    pub gruppenname: String,

    /// Die Herkunft der Gruppe
    pub herkunft: Herkunft,

    /// Bei einer indirekten Zuweisung, wird der Rollenname angegeben
    pub rollenname: Option<String>,

    /// Wann die Gruppe erstellt wurde
    pub erstellt_am: Time,

    /// Wann die Gruppe geaendert wurde
    pub geaendert_am: Time,
}

#[ComplexObject]
impl GruppeAnsicht {}
